var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));

// src/utils.ts
import * as ts from "typescript";
import path from "path";
function loadTsconfig(tsconfigPath) {
  const configFile = ts.readConfigFile(tsconfigPath, ts.sys.readFile);
  const configFileContent = ts.parseJsonConfigFileContent(
    configFile.config,
    ts.sys,
    path.dirname(tsconfigPath)
  );
  return configFileContent;
}
function getFileLoc(diagnostic) {
  if (diagnostic.file) {
    const { line, character } = ts.getLineAndCharacterOfPosition(
      diagnostic.file,
      diagnostic.start
    );
    return `${diagnostic.file.fileName}:${line + 1}:${character + 1}`;
  }
  return "";
}
var transpileOutputBundleList = [];
function useDtsOnly({
  compilerOptions
}) {
  return () => {
    return {
      transformSourceFile(node) {
        return node;
      },
      transformBundle(node) {
        transpileOutputBundleList.splice(0, transpileOutputBundleList.length);
        node.sourceFiles.forEach((sourceFile) => {
          const transpileOutput = ts.transpileDeclaration(sourceFile.text, {
            fileName: sourceFile.fileName,
            compilerOptions
          });
          transpileOutputBundleList.push({
            transpileOutput,
            sourceFile
          });
        });
        return node;
      }
    };
  };
}
function formatSourceFileTextByNamespace(newDeclarationOutPutText, isNamespace) {
  return newDeclarationOutPutText.split("\n").map((line) => {
    if (/\.\/|\.\.\//.test(line) && /from/.test(line)) {
      return `    // ${line}`;
    } else if (isNamespace) {
      return "    " + line.replace(" declare", "");
    }
    return line;
  }).join("\n");
}

// src/tsc.ts
import * as ts2 from "typescript";
import { logger } from "node-logger-plus";
function emitDts({
  rawCompilerOptions: rawCompilerOptions2,
  fileNames: fileNames2,
  rslibDtsOptions
}, rspackOptions) {
  const compilerOptions = __spreadProps(__spreadValues({}, rawCompilerOptions2), {
    noEmit: false,
    declaration: true,
    // 生成.d描述文件
    emitDeclarationOnly: true
    // 仅发布声明文件
  });
  const createFiles = {};
  const host = ts2.createCompilerHost(compilerOptions);
  host.writeFile = (fileName, contents) => {
    createFiles[fileName] = contents;
    if (rslibDtsOptions.only) {
      let newDeclarationOutPutText = transpileOutputBundleList.map((item) => {
        return item.transpileOutput.outputText;
      }).join("");
      const libraryName = rslibDtsOptions.name || rspackOptions.output.library.name;
      libraryName && logger.debug("libraryName:", libraryName);
      if (libraryName) {
        newDeclarationOutPutText = `export declare namespace ${libraryName} {
${formatSourceFileTextByNamespace(newDeclarationOutPutText, true)}
}`;
      }
      ts2.sys.writeFile(fileName, formatSourceFileTextByNamespace(newDeclarationOutPutText, false));
    } else {
      ts2.sys.writeFile(fileName, contents);
    }
  };
  const program = ts2.createProgram(fileNames2, compilerOptions, host);
  const emitResult = program.emit(void 0, void 0, void 0, void 0, {
    afterDeclarations: rslibDtsOptions.only ? [useDtsOnly({ compilerOptions })] : []
  });
  console.log("### JavaScript\n");
  console.log(fileNames2.join("\n"));
  console.log("### Type Definition\n");
  const allDiagnostics = ts2.getPreEmitDiagnostics(program).concat(emitResult.diagnostics);
  const diagnosticMessages = [];
  logger.success("TypeScript EmitDiagnostics closed successfully");
  for (const diagnostic of allDiagnostics) {
    const fileLoc = getFileLoc(diagnostic);
    const message = `${fileLoc} - ${"error"} ${`TS${diagnostic.code}:`} ${ts2.flattenDiagnosticMessageText(
      diagnostic.messageText,
      host.getNewLine()
    )}`;
    diagnosticMessages.push(message);
  }
  if (diagnosticMessages.length) {
    logger.error(
      `Failed to emit declaration files.`
    );
    for (const message of diagnosticMessages) {
      logger.error(message);
    }
  }
}

// src/config/index.ts
var rspackDtsPluginOptions = {
  dts: true,
  only: false,
  outputDir: "./dist/types",
  outFile: "./dist/types/index",
  rootDir: "src",
  name: void 0
};
function mergeRspackDtsPluginOptions(...options) {
  return Object.assign({}, rspackDtsPluginOptions, ...options);
}

// src/dts.ts
import { logger as logger2 } from "node-logger-plus";
var { options: rawCompilerOptions, fileNames } = loadTsconfig("tsconfig.json");
function buildDts(options, rspackOptions) {
  var _a, _b, _c;
  const rslibDtsOptions = mergeRspackDtsPluginOptions(options);
  const { dts, only, outputDir, rootDir, outFile } = rslibDtsOptions;
  if (!dts) {
    return;
  }
  if (only) {
    rawCompilerOptions.outFile = (_a = rawCompilerOptions.outFile) != null ? _a : outFile;
    rawCompilerOptions.declarationDir = null;
  } else {
    rawCompilerOptions.declarationDir = (_b = rawCompilerOptions.declarationDir) != null ? _b : outputDir;
    rawCompilerOptions.rootDir = (_c = rawCompilerOptions.rootDir) != null ? _c : rootDir;
  }
  logger2.info(`Building DTS configuration`);
  emitDts({
    rawCompilerOptions,
    fileNames,
    rslibDtsOptions
  }, rspackOptions);
}

// src/index.ts
var RspackDtsPlugin = class {
  constructor(options = {}) {
    this.options = options;
  }
  apply(compiler) {
    compiler.hooks.initialize.tap("initialize", () => {
      buildDts(this.options, compiler.options);
    });
  }
};
export {
  RspackDtsPlugin,
  buildDts
};
